/* * Cadence MACB/GEM Ethernet Controller driver
 *
 * Copyright (C) 2004-2006 Atmel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#define pr_fmt(fmt) KBUILD_MODNAME ": " fmt
#include <linux/clk.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/circ_buf.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/io.h>
#include <linux/gpio.h>
#include <asm/mman.h>
#include <linux/interrupt.h>
#include <linux/etherdevice.h>
#include <linux/dma-mapping.h>
#include <linux/platform_data/macb.h>
#include <linux/phy.h>
#include <linux/if_arp.h>
#include <linux/of.h>
#include <linux/cdev.h>
#include <linux/of_device.h>
#include <linux/of_mdio.h>
#include <linux/of_net.h>

#ifdef CONFIG_MACB_DEVICE_POLL
#include <linux/netdev_poll.h>
#endif

#ifdef CONFIG_FPGA_PERIPHERAL
#include <linux/rtnetlink.h>
#include <misc/fpgaperipheral.h>
#endif

#include "core.h"
#include "regs.h"
#include "common.h"

// Linux kernel time starts from 1970 while LabVIEW time starts from 1904.
// Right now EtherCAT time aligns with LabVIEW time because of the nanosecond engine.
// Following epoch_offset is seconds from 1904 ~ 1970.
// We reserved an IOCTL command to re-specify this offset.
#define EPOCH_OFFSET     2082844800ULL

#define DATA_BLOCK_OFFSET      26
#define PHY_TIMEOUT              3000  // Timeout for wait PHY to be in running state (ms).
#define MMAP_ERROR_LOW           -130
#define MMAP_ERROR_HIGH          -1

#define NWCTRL_STARTTX_MASK      0x00000200 // Start tx (tx_go)
// Number of receive buffer bytes as a unit, this is HW setup
#define RX_BUF_UNIT              64
// DMA buffer descriptors must be aligned on a 4-byte boundary.
#define ALIGNMENT_BD             8

// Transmission timeout is 3 seconds.
#define TX_TIMEOUT      (3*HZ)
#define ETHER_ADDR_LEN          6

#define NIECAT_ETHER_ADDR_STR_LEN (ETHER_ADDR_LEN * 3)

#define NIECAT_BDRING_NEXT(ringptr, bdptr)        \
   (((u32)(bdptr) >= (ringptr)->lastbdaddr) ?      \
   (struct niecat_bd*)(ringptr)->firstbdaddr :   \
   (struct niecat_bd*)((u32)(bdptr) + (ringptr)->separation))
#define NIECAT_BD_TO_INDEX(ringptr, bdptr) \
   (((u32)bdptr - (u32)(ringptr)->firstbdaddr) / (ringptr)->separation)
#define NIECAT_RING_SEEKAHEAD(ringptr, bdptr, numbd)          \
{                                                              \
   u32 addr = (u32)bdptr;                                      \
   addr += ((ringptr)->separation * numbd);                    \
   if ((addr > (ringptr)->lastbdaddr) || ((u32)bdptr > addr))  \
   {                                                           \
      addr -= (ringptr)->length;                               \
   }                                                           \
   bdptr = (struct niecat_bd*)addr;                           \
}

static const char* const kNIECATModuleName = "niecatk";
static const char* const kNIECATClassName = "ethercat";
static const char* const kNIECATDevNameFmt = "niecat%u";

#define MAX_ECAT_DEV_NUM 64
static DEFINE_MUTEX(ecat_mtx);
static DECLARE_BITMAP(ecat_minors, MAX_ECAT_DEV_NUM);

static int _fake_open(struct net_device *devp)
{
   return -1;
}

static int _fake_close(struct net_device *devp)
{
   return 0;
}

static struct net_device_ops niecat_netdev_ops = {
   .ndo_open = _fake_open,
   .ndo_stop = _fake_close,
   .ndo_start_xmit = NULL,
   .ndo_set_rx_mode = NULL,
   .ndo_get_stats = NULL,
   .ndo_do_ioctl = NULL,
   .ndo_validate_addr = NULL,
   .ndo_change_mtu = NULL,
   .ndo_set_mac_address = NULL,
#ifdef CONFIG_NET_POLL_CONTROLLER
   .ndo_poll_controller = NULL,
#endif
};

bool niecat_running(struct niecat *ep)
{
   if(test_bit(NIECAT_STATE_ECAT_RUNNING, &ep->flags))
	return true;
   else
	return false;
}

void niecat_init_txrx(struct niecat *ep)
{
   // Initialize TX and RX buffers
   niecat_writel(ep, RBQP, ep->erx_ring.physbaseaddr);
   niecat_writel(ep, TBQP, ep->etx_ring.physbaseaddr);
}

u32 niecat_get_dmacfg(void)
{
   u32 regval;

   regval  = (((NIECAT_RX_BUF_SIZE / RX_BUF_UNIT) +
             ((NIECAT_RX_BUF_SIZE % RX_BUF_UNIT) ? 1 : 0)) << NIECAT_GEM_RXBS_OFFSET);
   regval |= NIECAT_GEM_F(RXBMS);
   regval |= NIECAT_GEM_BIT(TXPBMS);
   regval |= NIECAT_GEM_BIT(TXCOEN);

#if defined(__LITTLE_ENDIAN)
   regval &= ~NIECAT_GEM_BIT(ENDIA);
#elif defined(__BIG_ENDIAN)
   regval |= NIECAT_GEM_BIT(ENDIA);
#else
#error No defined Constant Symbol __BIG_ENDIAN or __LITTLE_ENDIAN
#endif
   regval |= NIECAT_GEM_BIT(INCR16);
   return regval;
}

static int _mmap(struct file* filp, struct vm_area_struct* vma)
{
   struct niecat* ep = filp->private_data;

   if (!ep)
     return -ENODEV;

   if(remap_pfn_range(
      vma,
      vma->vm_start,
      vma->vm_pgoff,
      vma->vm_end-vma->vm_start,
      vma->vm_page_prot))
   {
      return -EAGAIN;
   }

   return 0;
}

int ecat_getbuf(struct niecat *ep, void __user *arg)
{
   int res = 0;
   unsigned long physaddr;

   if (!ep->send_recv_buf)
   {
      niecat_debug("NI EtherCAT niecat: can't get enough pages for send/recv buffer!\n");
      return -ENOMEM;
   }

   physaddr = virt_to_phys((void*)ep->send_recv_buf);
   if (copy_to_user(arg, &physaddr, sizeof(physaddr)))
   {
      res = -EFAULT;
   }

   return res;
}

static int _init_send_recv_buf(struct file* filp)
{
   struct niecat* ep = filp->private_data;

   ep->send_recv_buf = (u8*)__get_free_pages(GFP_KERNEL, get_order(NIECAT_MMAP_SIZE));
   if (!ep->send_recv_buf)
   {
      niecat_debug("NI EtherCAT niecat: can't get enough pages for send/recv buffer!\n");
      return -ENOMEM;
   }

   // Make sure user address space is still valid before doing this.
   if (current->mm == NULL)
   {
      niecat_debug("NI EtherCAT niecat: user mode address space is already destroyed!\n");
      return -1;
   }

   return 0;
}

static void _clean_send_recv_buf(struct file* filp)
{
   struct niecat* ep = filp->private_data;

   if (ep->send_recv_buf)
   {
      free_pages((unsigned long)ep->send_recv_buf, get_order(NIECAT_MMAP_SIZE));
      ep->send_recv_buf = NULL;
   }
}

static int _cdev_open(struct inode* inodep, struct file* filp)
{
   int res = 0;
   struct niecat* ep = container_of(inodep->i_cdev, struct niecat, char_dev);
   filp->private_data = ep;

   res = _init_send_recv_buf(filp);
   if (res < 0)
   {
      niecat_debug(KERN_ERR "%s Unable to initialize send and receive buffers, res %d\n", ep->dev->name, res);
      _clean_send_recv_buf(filp);
   }

#ifdef CONFIG_FPGA_PERIPHERAL
   rtnl_lock();
#endif
   if (!test_bit(NIECAT_STATE_ECAT_RUNNING, &ep->flags))
   {
      niecat_debug("Bringup not running\n");
#ifdef CONFIG_FPGA_PERIPHERAL
      // If we're being brought up while the FPGA is being reprogrammed, we can just set the state.
      // The interface will be brought up when the FPGA is back up.
      if (*ep->fpga_down)
      {
         set_bit(NIECAT_STATE_ECAT_RUNNING, &ep->flags);
      }
      else
#endif
      {
         niecat_debug("niecat_bringup devce name:%s\n",ep->dev->name);
         res = niecat_bringup(ep);
         if (res == 0)
         {
            niecat_debug("Bringup sucessfully\n");
            set_bit(NIECAT_STATE_ECAT_RUNNING, &ep->flags);
         }
      }
   }
   else
   {
      niecat_debug("%s is already running. Can NOT been set to running again!\n", ep->dev->name);
   }
#ifdef CONFIG_FPGA_PERIPHERAL
   rtnl_unlock();
#endif
   
   return res;
}

static void _clean_rings(struct niecat* ep)
{
   int i;

   for (i = 0; i < RECV_BD_CNT; i++)
   {
      if (ep->rx_ring_info && ep->rx_ring_info[i].dma_buf)
      {
         dma_unmap_single(
            &ep->pdev->dev,
            ep->rx_ring_info[i].dma_addr,
            NIECAT_RX_BUF_SIZE,
            DMA_FROM_DEVICE);
         kfree(ep->rx_ring_info[i].dma_buf);

         ep->rx_ring_info[i].dma_buf = NULL;
         ep->rx_ring_info[i].dma_addr = 0;
      }
   }

   for (i = 0; i < SEND_BD_CNT; i++)
   {
      if (ep->tx_ring_info && ep->tx_ring_info[i].dma_buf)
      {
         dma_unmap_single(
            &ep->pdev->dev,
            ep->tx_ring_info[i].dma_addr,
            NIECAT_TX_BUF_SIZE,
            DMA_TO_DEVICE);
         kfree(ep->tx_ring_info[i].dma_buf);

         ep->tx_ring_info[i].dma_buf = NULL;
         ep->tx_ring_info[i].dma_addr = 0;
      }
   }
}

static void _descriptor_free(struct niecat* ep)
{
   int size;

   // kfree(NULL) is safe, no need to check here
   kfree(ep->tx_ring_info);
   ep->tx_ring_info = NULL;
   kfree(ep->rx_ring_info);
   ep->rx_ring_info = NULL;

   size = RECV_BD_CNT * sizeof(struct niecat_bd);
   if (ep->rx_bd_info.dma_buf)
   {
      dma_free_coherent(
         &ep->pdev->dev,
         size,
         ep->rx_bd_info.dma_buf,
         ep->rx_bd_info.dma_addr);
      ep->rx_bd_info.dma_buf = NULL;
   }

   size = SEND_BD_CNT * sizeof(struct niecat_bd);
   if (ep->tx_bd_info.dma_buf)
   {
      dma_free_coherent(
         &ep->pdev->dev,
         size,
         ep->tx_bd_info.dma_buf,
         ep->tx_bd_info.dma_addr);
      ep->tx_bd_info.dma_buf = NULL;
   }
}

static int _cdev_release(struct inode* inode, struct file* filp)
{
   int res = 0;
   struct niecat* ep = filp->private_data;

#ifdef CONFIG_FPGA_PERIPHERAL
   rtnl_lock();
#endif
   if (test_bit(NIECAT_STATE_ECAT_RUNNING, &ep->flags))
   {
      clear_bit(NIECAT_STATE_ECAT_RUNNING, &ep->flags);

#ifdef CONFIG_FPGA_PERIPHERAL
      // If we're being brought down while the FPGA is being programmed,
      // the interface is already down. We can just return.
      if (!(*ep->fpga_down))
#endif
      {
         res = niecat_bringdown(ep);
      }
   }
#ifdef CONFIG_FPGA_PERIPHERAL
   rtnl_unlock();
#endif

   _clean_send_recv_buf(filp);

   return res;
}

static int _send(struct net_device* devp, const struct send_data_block* data_block)
{
   struct niecat* ep = netdev_priv(devp);
   struct niecat_bd* bdptr;
   u32 regval, bdidx;
#ifdef WORK_AROUND_L2_CACHE_COHERENT_ISSUE
   phys_addr_t current_bd_phys;
#endif

   if (!test_bit(NIECAT_STATE_UP, &ep->flags))
   {
      return -ENETDOWN;
   }

   bdptr = ep->etx_ring.freehead;
   bdidx = NIECAT_BD_TO_INDEX(&ep->etx_ring, bdptr);

#ifdef WORK_AROUND_L2_CACHE_COHERENT_ISSUE
   // DMA address is bus address. It's different from physical address on some architecture.
   // But on ARM there is no offset between bus address and physical address.
   // So we can use the dma address as physical address here.
   current_bd_phys = ep->tx_bd_info.dma_addr + bdidx * sizeof(struct niecat_bd);
   outer_inv_range(current_bd_phys, current_bd_phys + sizeof(struct niecat_bd));
#endif
   regval = ACCESS_ONCE(bdptr->ctrl);
   if (!(regval & NIECAT_BIT(TX_USED)))
   {
      return -ENOSPC;
   }
   NIECAT_RING_SEEKAHEAD(&ep->etx_ring, ep->etx_ring.freehead, 1);

   memcpy(ep->tx_ring_info[bdidx].dma_buf, data_block->data, data_block->data_len);
   dma_sync_single_for_device(
      &ep->pdev->dev,
      ep->tx_ring_info[bdidx].dma_addr,
      data_block->data_len,
      DMA_TO_DEVICE);

   // Preserve only critical status bits. Packet is NOT to be committed to hardware
   // at this time. NIECAT_TXBUF_USED_MASK bit is cleared.
   regval &= NIECAT_BIT(TX_WRAP);
   // update length field
   regval |= ((regval & ~NIECAT_GEM_F(TX_FRMLEN)) | data_block->data_len);
   regval |= NIECAT_BIT(TX_LAST);
   bdptr->ctrl = regval;
   wmb();

   ep->stats.tx_packets += 1;
   ep->stats.tx_bytes += data_block->data_len;
#ifdef DEBUG_VERBOSE_TX
   printk(KERN_DEBUG "GEM: TX BD index %d, BDptr %p, BD_STAT 0x%08x\n",
      bdidx, bdptr, bdptr->ctrl);
#endif

   regval = niecat_readl(ep, NCR);
   niecat_writel(ep, NCR, (regval | NIECAT_BIT(TSTART)));
   wmb();

   return 0;
}

static int _descriptor_init(struct niecat* ep)
{
   int size;

   size = SEND_BD_CNT * sizeof(struct niecat_dma_info);
   ep->tx_ring_info = kzalloc(size, GFP_KERNEL);
   if (!ep->tx_ring_info)
   {
      goto err_out;
   }
   size = RECV_BD_CNT * sizeof(struct niecat_dma_info);
   ep->rx_ring_info = kzalloc(size, GFP_KERNEL);
   if (!ep->rx_ring_info)
   {
      goto err_out;
   }

   size = RECV_BD_CNT * sizeof(struct niecat_bd);
   ep->rx_bd_info.dma_buf = dma_alloc_coherent(
      &ep->pdev->dev,
      size,
      &(ep->rx_bd_info.dma_addr),
      GFP_KERNEL);
   if (!ep->rx_bd_info.dma_buf)
   {
      goto err_out;
   }
   dev_dbg(&ep->pdev->dev, "RX ring %d bytes at 0x%x mapped %p\n",
      size, ep->rx_bd_info.dma_addr, ep->rx_bd_info.dma_buf);

   size = SEND_BD_CNT * sizeof(struct niecat_bd);
   ep->tx_bd_info.dma_buf = dma_alloc_coherent(
      &ep->pdev->dev,
      size,
      &(ep->tx_bd_info.dma_addr),
      GFP_KERNEL);
   if (!ep->tx_bd_info.dma_buf)
   {
      goto err_out;
   }
   dev_dbg(&ep->pdev->dev, "TX ring %d bytes at 0x%x mapped %p\n",
      size, ep->tx_bd_info.dma_addr, ep->tx_bd_info.dma_buf);

   return 0;

err_out:
   _descriptor_free(ep);
   return -ENOMEM;
}

static int _setup_recv_buffers(struct niecat* ep)
{
   struct niecat_bd* bdptr;
   u32 dma_addr, regval;
   u8* dma_buf;
   int bd_counter;
   int bdidx;

   bdptr = (struct niecat_bd*)(ep->erx_ring.firstbdaddr);
   for (bd_counter = 0; bd_counter < RECV_BD_CNT; ++bd_counter)
   {
      dma_buf = kmalloc(NIECAT_RX_BUF_SIZE, GFP_KERNEL);
      if (dma_buf == NULL)
      {
         niecat_debug("%s allocates Rx DMA buffers failed.\n", ep->dev->name);
         return -ENOMEM;
      }
      dma_addr = dma_map_single(
         &ep->pdev->dev,
         dma_buf,
         NIECAT_RX_BUF_SIZE,
         DMA_FROM_DEVICE);

      bdidx = NIECAT_BD_TO_INDEX(&ep->erx_ring, bdptr);
      ep->rx_ring_info[bdidx].dma_addr = dma_addr;
      ep->rx_ring_info[bdidx].dma_buf = dma_buf;

      regval = (bdptr->addr & ~NIECAT_F(RX_WADDR)) | dma_addr;
      regval &= ~NIECAT_BIT(RX_USED);
      bdptr->addr = regval;

      bdptr = NIECAT_BDRING_NEXT(&ep->erx_ring, bdptr);
   }
   wmb();

   return 0;
}

static int _setup_send_buffers(struct niecat* ep)
{
   struct niecat_bd* bdptr;
   u32 dma_addr;
   u8* dma_buf;
   int bd_counter;
   int bdidx;

   bdptr = (struct niecat_bd*)(ep->etx_ring.firstbdaddr);
   for (bd_counter = 0; bd_counter < SEND_BD_CNT; ++bd_counter)
   {
      dma_buf = kmalloc(NIECAT_TX_BUF_SIZE, GFP_KERNEL);
      if (dma_buf == NULL)
      {
         niecat_debug("%s allocates Tx DMA buffers failed.\n", ep->dev->name);
         return -ENOMEM;
      }

      dma_addr = dma_map_single(
         &ep->pdev->dev,
         dma_buf,
         NIECAT_TX_BUF_SIZE,
         DMA_TO_DEVICE);

      bdidx = NIECAT_BD_TO_INDEX(&ep->etx_ring, bdptr);
      ep->tx_ring_info[bdidx].dma_addr = dma_addr;
      ep->tx_ring_info[bdidx].dma_buf = dma_buf;

      bdptr->addr = dma_addr;

      bdptr = NIECAT_BDRING_NEXT(&ep->etx_ring, bdptr);
   }
   wmb();

   return 0;
}

static int _setup_ring(struct niecat* ep)
{
   int i, rc;
   struct niecat_bd* bdptr;

   ep->erx_ring.separation   = (sizeof(struct niecat_bd) + (ALIGNMENT_BD - 1)) & ~(ALIGNMENT_BD - 1);
   ep->erx_ring.physbaseaddr = ep->rx_bd_info.dma_addr;
   ep->erx_ring.firstbdaddr  = (u32)(ep->rx_bd_info.dma_buf);
   ep->erx_ring.lastbdaddr   = (u32)(ep->rx_bd_info.dma_buf + (RECV_BD_CNT - 1) * sizeof(struct niecat_bd));
   ep->erx_ring.length       = ep->erx_ring.lastbdaddr - ep->erx_ring.firstbdaddr + ep->erx_ring.separation;
   ep->erx_ring.freehead     = (struct niecat_bd*)(ep->rx_bd_info.dma_buf);
   ep->erx_ring.hwhead       = (struct niecat_bd*)(ep->rx_bd_info.dma_buf);

   // Setup RX BD ring structure and populate buffer address.
   bdptr = (struct niecat_bd*)ep->erx_ring.firstbdaddr;
   for (i = 0; i < (RECV_BD_CNT - 1); i++)
   {
      bdptr->addr = 0;
      bdptr->ctrl = 0;
      bdptr = NIECAT_BDRING_NEXT(&ep->erx_ring, bdptr);
   }
   // wrap bit set for last BD, bdptr is moved to last here
   bdptr->addr = NIECAT_BIT(RX_WRAP);
   bdptr->ctrl = 0;

   // Allocate RX buffs; set descriptor buffer addresses
   rc = _setup_recv_buffers(ep);
   if (rc)
   {
      goto err_out;
   }

   ep->etx_ring.separation   = (sizeof(struct niecat_bd) + (ALIGNMENT_BD - 1)) & ~(ALIGNMENT_BD - 1);
   ep->etx_ring.physbaseaddr = ep->tx_bd_info.dma_addr;
   ep->etx_ring.firstbdaddr  = (u32)(ep->tx_bd_info.dma_buf);
   ep->etx_ring.lastbdaddr   = (u32)(ep->tx_bd_info.dma_buf + (SEND_BD_CNT - 1) * sizeof(struct niecat_bd));
   ep->etx_ring.length       = ep->etx_ring.lastbdaddr - ep->etx_ring.firstbdaddr + ep->etx_ring.separation;
   ep->etx_ring.freehead     = (struct niecat_bd*)(ep->tx_bd_info.dma_buf);
   ep->etx_ring.hwhead       = (struct niecat_bd*)(ep->tx_bd_info.dma_buf);

   // Setup TX BD ring structure and assert used bit initially.
   bdptr = (struct niecat_bd*)ep->etx_ring.firstbdaddr;
   for (i = 0; i < (SEND_BD_CNT - 1); i++)
   {
      bdptr->addr = 0;
      bdptr->ctrl = NIECAT_BIT(TX_USED);
      bdptr = NIECAT_BDRING_NEXT(&ep->etx_ring, bdptr);
   }
   // wrap bit set for last BD, bdptr is moved to last here
   bdptr->addr = 0;
   bdptr->ctrl = (NIECAT_BIT(TX_WRAP) | NIECAT_BIT(TX_USED));

   // Allocate TX buffs; set descriptor buffer addresses
   rc = _setup_send_buffers(ep);
   if (rc)
   {
      goto err_out;
   }

   return 0;

err_out:
   _clean_rings(ep);
   return rc;
}

static unsigned int _receive(
   struct niecat* ep,
   unsigned int budget,
   struct recv_data_block* data_block,
   unsigned int work_done)
{
   struct niecat_bd* bdptr;
   u32 totalSize = 0;
   u32 packets = 0;
   u32 bdidx = 0;
   u32 len;
   u32 addr, ctrl;
#ifdef WORK_AROUND_L2_CACHE_COHERENT_ISSUE
   phys_addr_t current_bd_phys;
#endif

   while (packets < budget)
   {
      bdptr = ep->erx_ring.hwhead;
      bdidx = NIECAT_BD_TO_INDEX(&ep->erx_ring, bdptr);

#ifdef WORK_AROUND_L2_CACHE_COHERENT_ISSUE
      // DMA address is bus address. It's different from physical address on some architecture.
      // But on ARM there is no offset between bus address and physical address.
      // So we can use the dma address as physical address here.
      current_bd_phys = ep->rx_bd_info.dma_addr + bdidx * sizeof(struct niecat_bd);
      outer_inv_range(current_bd_phys, current_bd_phys + sizeof(struct niecat_bd));
#endif
      addr = ACCESS_ONCE(bdptr->addr);
      // Read the status word to see if BD has been processed.
      if (!(addr & NIECAT_BIT(RX_USED)))
      {
         niecat_debug("BD has been processed, addr:%x\n", addr);
         break;
      }
      NIECAT_RING_SEEKAHEAD(&ep->erx_ring, ep->erx_ring.hwhead, 1);

      ctrl = ACCESS_ONCE(bdptr->ctrl);
      niecat_debug("GEM: %s: RX BD index %d, BDptr %p, BD_STAT 0x%08x\n",
         __func__, bdidx, bdptr, ctrl);

      if (ctrl & NIECAT_BIT(RX_BAD))
      {
         niecat_debug("EtherCAT network driver detected a frame with bad FCS\n");
      }
      else
      {
         // the packet length
         len = ctrl & NIECAT_F(RX_FRMLEN);
         data_block[work_done + packets].data_len = len;
         dma_sync_single_for_cpu(
            &ep->pdev->dev,
            ep->rx_ring_info[bdidx].dma_addr,
            len,
            DMA_FROM_DEVICE);
         memcpy((void*)data_block[work_done + packets].data, ep->rx_ring_info[bdidx].dma_buf, len);

         totalSize += len;
         ++packets;
      }

      // clear the new data mask from the BD.
      bdptr->ctrl = 0;
      bdptr->addr = (addr & ~NIECAT_BIT(RX_USED));
      wmb();
   }

   ep->stats.rx_packets += packets;
   ep->stats.rx_bytes += totalSize;

   return packets;
}

static int _receive_poll(
   struct net_device* ndev,
   struct recv_data_block* data_block,
   unsigned int budget)
{
   struct niecat* ep = netdev_priv(ndev);
   unsigned int work_done = 0;
   unsigned int temp_work_done;
   u32 regval;

   if (!test_bit(NIECAT_STATE_UP, &ep->flags))
   {
      return -ENETDOWN;
   }

   while (work_done < budget)
   {
      regval = niecat_readl(ep, RSR);
      niecat_writel(ep, RSR, regval);
      wmb();

      temp_work_done = _receive(ep, budget - work_done, data_block, work_done);

      if (temp_work_done != 0)
      {
         work_done += temp_work_done;
      }
      else
      {
         break;
      }
   }

   return work_done;
}

void niecat_init_ecat(
   struct niecat *ep,
   struct resource *regs,
   spinlock_t *lock,
#ifdef CONFIG_FPGA_PERIPHERAL
   bool *fpga_down,
#endif
   struct net_device *dev,
   struct platform_device *pdev,
   niecat_init_hw_cb init_hw,
   niecat_mii_probe_cb mii_probe,
   niecat_reset_hw_cb reset_hw,
   niecat_get_hwaddr_cb get_hwaddr)
{
   dev->netdev_ops = &niecat_netdev_ops;
   dev->watchdog_timeo = TX_TIMEOUT;
   ep->regs = regs;
   ep->lock = lock;
#ifdef CONFIG_FPGA_PERIPHERAL
   ep->fpga_down = fpga_down;
#endif
   ep->dev = dev;
   ep->pdev = pdev;
   ep->init_hw_cb = init_hw;
   ep->mii_probe_cb = mii_probe;
   ep->reset_hw_cb = reset_hw;
   ep->get_hwaddr_cb = get_hwaddr;
   ep->epoch_offset = EPOCH_OFFSET;
}

static int _start_phy(struct niecat *ep)
{
   u64 start_time, current_time;
   struct timespec kernel_tp;
   struct phy_device *phydev = ep->phy_dev;
   // When the transmission speed defined in the driver is changed on the fly,
   // the link will go down temporarily(phy state is PHY_NOLINK). 
   // After the new speed goes into effect, the link will go up again.
   // Since this NOLINK status could be recoverable, so we add some sleep time(700 ms) to wait the
   // link go up. If the timeout happens, we thought the network cable is unplugged.
   const u8 kNolinkWaitCount = 70;
   u8 nolink_count = 0;

   phy_start(phydev);
   cancel_delayed_work_sync(&phydev->state_queue);
   queue_delayed_work(system_power_efficient_wq, &phydev->state_queue,0);

   ktime_get_real_ts(&kernel_tp);
   start_time = 1000000000ULL * kernel_tp.tv_sec + kernel_tp.tv_nsec;
   while (phydev->state != PHY_RUNNING)
   {
      if (phydev->state == PHY_NOLINK)
      {
          ++nolink_count;
          if (nolink_count > kNolinkWaitCount)
          {
              break;
          }
      }
      else
      {
          nolink_count = 0;
          ktime_get_real_ts(&kernel_tp);
          current_time = 1000000000ULL * kernel_tp.tv_sec + kernel_tp.tv_nsec;
          if ((current_time - start_time)/1000000 > PHY_TIMEOUT)
          {
              niecat_debug("%s: Timeout while waiting PHY to be running!\n", ep->dev->name);
              return -1;
          }
      }

      msleep(10);
   }

   niecat_debug("%s, phy_addr %s, phy_id 0x%08x\n",
         ep->dev->name, phydev_name(phydev), phydev->phy_id);

   niecat_debug("%s, attach [%s] phy driver\n", ep->dev->name,
         phydev->drv->name);

   return 0;
}

u32 niecat_get_initcfg(void)
{
   u32 config;
   config = NIECAT_BIT(FD);
   config |= NIECAT_BIT(RXCHKS);
   config |= NIECAT_BIT(PAE);
   config |= NIECAT_BIT(PSE);
   config |= NIECAT_BIT(SPD);
   return config;
}

int niecat_bringup(struct niecat* ep)
{
   int res = 0;
   unsigned long flags;

   dev_dbg(&ep->pdev->dev, "open\n");
   if (!is_valid_ether_addr(ep->dev->dev_addr))
   {
      return  -EADDRNOTAVAIL;
   }

   res = _descriptor_init(ep);
   if (res)
   {
      netdev_dbg(ep->dev, "Unable to allocate DMA memory, res %d\n", res);
      goto err_out;
   }

   res = _setup_ring(ep);
   if (res)
   {
      netdev_dbg(ep->dev, "Unable to setup BD rings, res %d\n", res);
      goto err_out;
   }

   ep->init_hw_cb(ep);

   res = _start_phy(ep);
   if (res)
   {
      netdev_dbg(ep->dev, "Unable to start PHY device, res %d\n", res);
      goto err_out;
   }

   set_bit(NIECAT_STATE_UP, &ep->flags);
   return 0;

err_out:
   spin_lock_irqsave(ep->lock, flags);
   ep->reset_hw_cb(ep);
   spin_unlock_irqrestore(ep->lock, flags);

   _clean_rings(ep);
   _descriptor_free(ep);
   return res;
}

int niecat_bringdown(struct niecat* ep)
{
   unsigned long flags;

   clear_bit(NIECAT_STATE_UP, &ep->flags);

   if (ep->phy_dev)
		phy_stop(ep->phy_dev);

   spin_lock_irqsave(ep->lock, flags);
   ep->reset_hw_cb(ep);
   spin_unlock_irqrestore(ep->lock, flags);

   _clean_rings(ep);
   _descriptor_free(ep);
   return 0;
}

static int _timestamp_send(struct net_device* devp, struct send_data_block* data_block)
{
   struct niecat* ep = netdev_priv(devp);
   unsigned long flags;
   u64 system_offset, timestamp;
   struct timespec kernel_tp;
   int res = 0;

   memcpy((void*)&system_offset, (void*)(data_block->data + DATA_BLOCK_OFFSET), 8);
   spin_lock_irqsave(ep->lock, flags);

   ktime_get_real_ts(&kernel_tp);
   timestamp = 1000000000ULL * (kernel_tp.tv_sec + ep->epoch_offset) + kernel_tp.tv_nsec - system_offset;

   memcpy((void*)(data_block->data + DATA_BLOCK_OFFSET), (void*)&timestamp, 8);
   res = _send(devp, data_block);

   spin_unlock_irqrestore(ep->lock, flags);

   return res;
}

static long _ioctl(struct file* filp, unsigned int cmd, unsigned long arg)
{
   int res = 0;
   struct niecat* ep = filp->private_data;
   struct net_device* net_dev = ep->dev;

   switch(cmd)
   {
   case IOC_GET_LINKSTATE:
      {
         if(ep->phy_dev)
         {
            int nVal = ep->phy_dev->link;
            put_user(nVal, (int *)arg);
         }
      }
      break;
   case IOC_SEND:
      {
         struct send_data_block* data_block = (struct send_data_block*)ep->send_recv_buf;
         res = _send(net_dev, data_block);
      }
      break;
   case IOC_RECV:
      {
         struct recv_data_block* data_block =
            (struct recv_data_block*)(ep->send_recv_buf + sizeof(struct send_data_block));
         res = _receive_poll(net_dev, data_block, arg);
      }
      break;
   case IOC_TIMESTAMP:
      {
         struct send_data_block* data_block = (struct send_data_block*)ep->send_recv_buf;
         res = _timestamp_send(net_dev, data_block);
      }
      break;
  case IOC_EPOCH_OFFSET:
      {
         ep->epoch_offset = arg;
      }
      break;
  case IOC_MACADDR:
      {
         u8 mac_addr[ETHER_ADDR_LEN];

         ep->get_hwaddr_cb(ep);
         memcpy(mac_addr, ep->dev->dev_addr, sizeof(mac_addr));
         niecat_debug(
                "mac_addr - %x:%x:%x:%x:%x:%x\n",
                mac_addr[0], mac_addr[1], mac_addr[2],
                mac_addr[3], mac_addr[4], mac_addr[5]);
         if (copy_to_user((void __user*)arg, mac_addr, sizeof(mac_addr)))
         {
            res = -EFAULT;
         }
      }
      break;
   case IOC_BRING_UP:
      {
         //Bring up operation has been moved to _cdev_open. Since the user mode and test cases are
         //still using this ioctl interface. We temporarily leave it blank here. 
      }
      break;
   case IOC_BRING_DOWN:
      {
         //Bring down operation has been moved to _cdev_release. The same as IOC_BRING_UP.
      }
      break;
   case IOC_GET_BUF:
      {
         res = ecat_getbuf(ep, (void __user *)arg);
         break;
      }
   default:
      {
         niecat_debug("NI EtherCAT niecat: unsupported IOCTL command = 0x%.08X\n", cmd);
         res = -EOPNOTSUPP;
      }
   }

   return res;
}

static struct file_operations niecat_fops =
{
   .owner = THIS_MODULE,
   .unlocked_ioctl = _ioctl,
   .mmap = _mmap,
   .open = _cdev_open,
   .release = _cdev_release,
};

static int _ecat_major = 0;
static struct class* _ecat_class = NULL;

void niecat_remove_cdev(struct niecat *ep)
{
    dev_t dev = ep->char_dev.dev;
    int i = MINOR(dev);
    // Remove character device
    clear_bit(NIECAT_STATE_UP, &ep->flags);
    clear_bit(NIECAT_STATE_ECAT_RUNNING, &ep->flags);
    device_destroy(_ecat_class, dev);
    cdev_del(&ep->char_dev);
    mutex_lock(&ecat_mtx);
    if (i < MAX_ECAT_DEV_NUM && i >= 0)
    {
       clear_bit(i, ecat_minors);
    }
    mutex_unlock(&ecat_mtx);
}

// niecat_setup_cdev - Setup character device informatin if a dev is probed.
// The device name includes master id information.
// ep: local device instance pointer
// Return 0 on success, negative value if error
int niecat_setup_cdev(struct niecat *ep)
{
   int result = 0;
   int i = 0;
   dev_t dev;
   struct device *class_dev;

   // Find out the master ID by current device mac address.
   ep->get_hwaddr_cb(ep);

   mutex_lock(&ecat_mtx);
   i = find_first_zero_bit(ecat_minors, MAX_ECAT_DEV_NUM);   
   if (i >= MAX_ECAT_DEV_NUM)
   {
      result = -EBUSY;
      goto out;
   }

   dev = MKDEV(_ecat_major, i);
   cdev_init(&ep->char_dev, &niecat_fops);
   ep->char_dev.owner = THIS_MODULE;
   ep->char_dev.ops = &niecat_fops;
   result = cdev_add(&ep->char_dev, dev, 1);
   if (result)
   {
      niecat_debug("NI EtherCAT niecat: add character device failed \n");
      goto out;
   }
   class_dev = device_create(_ecat_class, NULL, dev, NULL, "niecat%s", dev_name(&ep->pdev->dev));
   if (IS_ERR(class_dev))
   {
      netdev_dbg(ep->dev, "Fail to create ecat class\n");
      device_destroy(_ecat_class, ep->char_dev.dev);
      cdev_del(&ep->char_dev);
      result = PTR_ERR(class_dev);
      goto out;
   }

   set_bit(i, ecat_minors);
   mutex_unlock(&ecat_mtx);
   clear_bit(NIECAT_STATE_UP, &ep->flags);
   clear_bit(NIECAT_STATE_ECAT_RUNNING, &ep->flags);
   return 0;
out:
   mutex_unlock(&ecat_mtx);
   return result;
}

int niecat_init(void)
{
   int result = 0;
   dev_t dev;

   dev = MKDEV(0, 0);

   result = alloc_chrdev_region(&dev, 0, MAX_ECAT_DEV_NUM, kNIECATModuleName);

   if (result < 0)
   {
      niecat_debug("NI EtherCAT niecat: allocate ecat device failed \n");
      return result;
   }

   _ecat_major = MAJOR(dev);
   niecat_debug("niecat_major:%d\n", _ecat_major);
   _ecat_class = class_create(THIS_MODULE, kNIECATClassName);
   if (IS_ERR(_ecat_class))
   {
      niecat_debug("NI EtherCAT niecat: create device class failed \n");
      unregister_chrdev_region(MKDEV(_ecat_major, 0), MAX_ECAT_DEV_NUM);
      return result;
   }

   return 0;
};

void niecat_exit(void)
{
   class_destroy(_ecat_class);
   unregister_chrdev_region(MKDEV(_ecat_major, 0), MAX_ECAT_DEV_NUM);
}

// Retunrs whether the driver registration was successful or not.
