#ifndef _NI_ECAT_CORE_H
#define _NI_ECAT_CORE_H

#include <linux/cdev.h>
#include <linux/platform_device.h>
#include <linux/netdevice.h>

#define NIECAT_MAX_DEV_NUM              16
#define NIECAT_DRIVER_NAME              "ni_ethercat"
#define NIECAT_ECAT_DESCRIPTION         "National Instruments EtherCAT network driver"
#define NIECAT_NI_AUTHOR                "National Instruments"

// State bits that can be set in net_local flags member.
#define NIECAT_STATE_ECAT_RUNNING       0  // Network interface state knows by EtherCAT
#define NIECAT_STATE_UP                 1  // Network interface real state

#ifdef DEBUG
#define niecat_debug(x, a...) printk(KERN_INFO x, ## a)
#else
#define niecat_debug(x, a...)
#endif

struct niecat_dma_info
{
   dma_addr_t   dma_addr;  //dma address (bus address)
   void*        dma_buf;   //virtual address
};

// DMA buffer descriptor structure. Each BD is two words
struct niecat_bd
{
   u32 addr;
   u32 ctrl;
};
struct niecat_bdring
{
   u32 physbaseaddr;    // Physical address of 1st BD in list
   u32 firstbdaddr;     // Virtual address of 1st BD in list
   u32 lastbdaddr;      // Virtual address of last BD in the list
   u32 length;          // size of ring in bytes
   u32 separation;      // Number of bytes between the starting address of adjacent BDs
   struct niecat_bd* freehead; // First BD in the free group
   struct niecat_bd* hwhead;   // First BD in the work group
};

typedef void (*niecat_init_hw_cb)(void *);
typedef int (*niecat_mii_probe_cb)(void *);
typedef void (*niecat_reset_hw_cb)(void*);
typedef void (*niecat_get_hwaddr_cb)(void *);

struct niecat
{
   void __iomem            *regs;
   struct net_device       *dev;
   struct platform_device  *pdev;
   struct phy_device       *phy_dev;
   unsigned long           flags;
   struct net_device_stats stats;
   spinlock_t       *lock;
   struct cdev             char_dev;
   void*                   send_recv_buf;
   unsigned long           send_recv_mmap;
   struct niecat_bdring    erx_ring;
   struct niecat_bdring    etx_ring;
   struct niecat_dma_info*    rx_ring_info;
   struct niecat_dma_info*    tx_ring_info;

   struct niecat_dma_info     rx_bd_info;
   struct niecat_dma_info     tx_bd_info;
   unsigned long           epoch_offset;
   niecat_init_hw_cb       init_hw_cb;
   niecat_mii_probe_cb         mii_probe_cb;
   niecat_reset_hw_cb       reset_hw_cb;
   niecat_get_hwaddr_cb        get_hwaddr_cb;
#ifdef CONFIG_FPGA_PERIPHERAL
   bool*                   fpga_down;
#endif
};

void niecat_init_txrx(struct niecat *ep);
u32 niecat_get_dmacfg(void);
u32 niecat_get_initcfg(void);
void niecat_init_ecat(
   struct niecat *ep,
   struct resource *regs,
   spinlock_t *lock,
#ifdef CONFIG_FPGA_PERIPHERAL
	bool *fpga_down,
#endif
   struct net_device *dev,
   struct platform_device *pdev,
   niecat_init_hw_cb init_hw,
   niecat_mii_probe_cb mii_probe,
   niecat_reset_hw_cb reset_hw,
   niecat_get_hwaddr_cb get_hwaddr);
int niecat_bringup(struct niecat* ep);
int niecat_bringdown(struct niecat* ep);
void niecat_remove_cdev(struct niecat *ep);
int niecat_setup_cdev(struct niecat *ep);
int niecat_init(void);
void niecat_exit(void);
bool niecat_running(struct niecat *ep);
#endif /* _NI_ECAT_CORE_H */
